import { invalidateCacheByRouterState } from './invalidate-cache-by-router-state';
import { fillLazyItemsTillLeafWithHead } from './fill-lazy-items-till-leaf-with-head';
import { createRouterCacheKey } from './create-router-cache-key';
import { PAGE_SEGMENT_KEY } from '../../../shared/lib/segment';
/**
 * Common logic for filling cache with new sub tree data.
 */ function fillCacheHelper(newCache, existingCache, flightDataPath, prefetchEntry, fillLazyItems) {
    const isLastEntry = flightDataPath.length <= 5;
    const [parallelRouteKey, segment] = flightDataPath;
    const cacheKey = createRouterCacheKey(segment);
    const existingChildSegmentMap = existingCache.parallelRoutes.get(parallelRouteKey);
    if (!existingChildSegmentMap) {
        // Bailout because the existing cache does not have the path to the leaf node
        // Will trigger lazy fetch in layout-router because of missing segment
        return;
    }
    let childSegmentMap = newCache.parallelRoutes.get(parallelRouteKey);
    if (!childSegmentMap || childSegmentMap === existingChildSegmentMap) {
        childSegmentMap = new Map(existingChildSegmentMap);
        newCache.parallelRoutes.set(parallelRouteKey, childSegmentMap);
    }
    const existingChildCacheNode = existingChildSegmentMap.get(cacheKey);
    let childCacheNode = childSegmentMap.get(cacheKey);
    const cacheNodeSeedData = flightDataPath[3];
    if (isLastEntry) {
        if (cacheNodeSeedData && (!childCacheNode || !childCacheNode.lazyData || childCacheNode === existingChildCacheNode)) {
            const incomingSegment = cacheNodeSeedData[0];
            const rsc = cacheNodeSeedData[1];
            const loading = cacheNodeSeedData[3];
            childCacheNode = {
                lazyData: null,
                // When `fillLazyItems` is false, we only want to fill the RSC data for the layout,
                // not the page segment.
                rsc: fillLazyItems || incomingSegment !== PAGE_SEGMENT_KEY ? rsc : null,
                prefetchRsc: null,
                head: null,
                prefetchHead: null,
                loading,
                parallelRoutes: fillLazyItems && existingChildCacheNode ? new Map(existingChildCacheNode.parallelRoutes) : new Map()
            };
            if (existingChildCacheNode && fillLazyItems) {
                invalidateCacheByRouterState(childCacheNode, existingChildCacheNode, flightDataPath[2]);
            }
            if (fillLazyItems) {
                fillLazyItemsTillLeafWithHead(childCacheNode, existingChildCacheNode, flightDataPath[2], cacheNodeSeedData, flightDataPath[4], prefetchEntry);
            }
            childSegmentMap.set(cacheKey, childCacheNode);
        }
        return;
    }
    if (!childCacheNode || !existingChildCacheNode) {
        // Bailout because the existing cache does not have the path to the leaf node
        // Will trigger lazy fetch in layout-router because of missing segment
        return;
    }
    if (childCacheNode === existingChildCacheNode) {
        childCacheNode = {
            lazyData: childCacheNode.lazyData,
            rsc: childCacheNode.rsc,
            prefetchRsc: childCacheNode.prefetchRsc,
            head: childCacheNode.head,
            prefetchHead: childCacheNode.prefetchHead,
            parallelRoutes: new Map(childCacheNode.parallelRoutes),
            loading: childCacheNode.loading
        };
        childSegmentMap.set(cacheKey, childCacheNode);
    }
    fillCacheHelper(childCacheNode, existingChildCacheNode, flightDataPath.slice(2), prefetchEntry, fillLazyItems);
}
/**
 * Fill cache with rsc based on flightDataPath
 */ export function fillCacheWithNewSubTreeData(newCache, existingCache, flightDataPath, prefetchEntry) {
    fillCacheHelper(newCache, existingCache, flightDataPath, prefetchEntry, true);
}
export function fillCacheWithNewSubTreeDataButOnlyLoading(newCache, existingCache, flightDataPath, prefetchEntry) {
    fillCacheHelper(newCache, existingCache, flightDataPath, prefetchEntry, false);
}

//# sourceMappingURL=fill-cache-with-new-subtree-data.js.map